@@ -14,6 +14,7 @@ public:
     void mousePressEvent(QMouseEvent *event);
     void mouseMoveEvent(QMouseEvent *event);
     void mouseReleaseEvent(QMouseEvent *);
+    bool thumbnailEnabled;
     
 private:
     QPoint offset;
@@ -27,6 +28,8 @@ private:
     QLabel *nameLabel;
     QWidget *greenArrow;
     QWidget *redArrow;
+    QWidget *statusIndicator;
+    QLabel *thumbnail;
 
 signals:
     void acceptCall();
@@ -37,6 +40,12 @@ public slots:
     void setName(const QString &name);
     void acceptMousePress();
     void rejectMousePress();
+    void hideControls();
+    void showControls();
+    void setThumbnail(const QImage &image);
+
+private slots:
+    void readSettings();
 };
 
 #endif // MAINVIEW_H
